/// @addtogroup cymealg
/// @{

/////////////////////////////////////////////////////////////////////////////
///
/// @file
///
/// This header file contains an implementation of a few variants
/// of algorithms for the computation of the minimum cycle mean of a digraph.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2020 by Pawel Pilarczyk.
//
// This file is part of my research software. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started in January 2006. Last revision: August 23, 2019.


#ifndef _CYMEALG_CYCLEMEAN_H_
#define _CYMEALG_CYCLEMEAN_H_

// include selected header files from the CHomP library
#include "chomp/system/config.h"
#include "chomp/system/textfile.h"

namespace cymealg {

/// Runs the Karp algorithm for each strongly connected component
/// of the graph and returns the minimum mean cycle weight,
/// which can be negative.
/// As a byproduct, saves the transposed graph, if requested to.
template <class wType>
wType minMeanCycleWeight (const diGraph<wType> &g,
	diGraph<wType> *transposed = 0)
{
	// find the strongly connected components of the graph
	chomp::homology::multitable<int_t> compVertices, compEnds;
	bool copyweights = !!transposed;
	int_t countSCC = SCC (g, compVertices, compEnds,
		static_cast<diGraph<wType> *> (0), transposed, copyweights);
	if (countSCC <= 0)
		return 0;

	// compute the maximum size of each strongly connected component
	int_t maxCompSize = compEnds [0];
	for (int_t comp = 1; comp < countSCC; ++ comp)
	{
		int_t compSize = compEnds [comp] - compEnds [comp - 1];
		if (maxCompSize < compSize)
			maxCompSize = compSize;
	}

	// allocate arrays for weights and bit fields
	wType **F = new wType * [maxCompSize + 1];
	chomp::homology::BitField *finite =
		new chomp::homology::BitField [maxCompSize + 1];
	for (int_t i = 0; i <= maxCompSize; ++ i)
	{
		F [i] = new wType [maxCompSize];
		finite [i]. allocate (maxCompSize);
	}

	// compute the numbers of vertices in each component
	int_t nVertices (g. countVertices ());
	int_t *numbers = new int_t [nVertices];
	int_t *components = new int_t [nVertices];
	for (int_t i = 0; i < nVertices; ++ i)
		components [i] = -1;
	int_t offset = 0;
	for (int_t comp = 0; comp < countSCC; ++ comp)
	{
		int_t maxOffset = compEnds [comp];
		int_t pos = offset;
		for (; pos < maxOffset; ++ pos)
		{
			numbers [compVertices [pos]] = pos - offset;
			components [compVertices [pos]] = comp;
		}
		offset = pos;
	}

	// compute the minimum mean cycle weight for each component
	wType minWeight (0);
	for (int_t comp = 0; comp < countSCC; ++ comp)
	{
		int_t offset = comp ? compEnds [comp - 1] :
			static_cast<int_t> (0);
		int_t compSize = compEnds [comp] - offset;
		for (int_t i = 0; i <= compSize; ++ i)
			finite [i]. clearall (compSize);
		F [0] [0] = 0;
		finite [0]. set (0);

		// when to show the next progress indicator?
		int_t nextProgress = (compSize < 1000) ? (compSize + 1) : 0;
		short percentage = 0;

		// compute path progressions of given length
		for (int_t len = 1; len <= compSize; ++ len)
		{
			// process source vertices
			for (int_t i = 0; i < compSize; ++ i)
			{
				if (!finite [len - 1]. test (i))
					continue;
				wType prevWeight = F [len - 1] [i];
				int_t source = compVertices [offset + i];

				// process target vertices (and edges)
				int_t nEdges = g. countEdges (source);
				for (int edge = 0; edge < nEdges; ++ edge)
				{
					int_t target =
						g. getEdge (source, edge);
					if (components [target] != comp)
						continue;
					int_t j = numbers [target];
					wType newWeight = prevWeight +
						g. getWeight (source, edge);
					if (!finite [len]. test (j))
					{
						finite [len]. set (j);
						F [len] [j] = newWeight;
					}
					else if (newWeight < F [len] [j])
						F [len] [j] = newWeight;
				}
			}

			// show progress indicator if necessary
			if (len >= nextProgress)
			{
				chomp::homology::scon << std::setw (3) <<
					percentage << "%\b\b\b\b";
				++ percentage;
				nextProgress = (percentage * compSize) / 75;
			}
		}

		// compute the minimum mean cycle weight for this component
		nextProgress = (compSize < 1000) ? (compSize + 1) : 0;
		percentage = 0;
		wType minCompWeight (0);
		bool firstMin = true;
		for (int_t vert = 0; vert < compSize; ++ vert)
		{
			if (!finite [compSize]. test (vert))
				continue;
			bool firstAverage = true;
			wType maxAverage = 0;
			for (int_t first = 0; first < compSize; ++ first)
			{
				if (!finite [first]. test (vert))
					continue;
				wType average = (F [compSize] [vert] -
					F [first] [vert]) /
					(compSize - first);
				if (firstAverage)
				{
					maxAverage = average;
					firstAverage = false;
				}
				else if (maxAverage < average)
					maxAverage = average;
			}
			if (firstMin || (maxAverage < minCompWeight))
			{
				if (firstAverage)
					throw "Bug in Karp's algorithm";
				minCompWeight = maxAverage;
				firstMin = false;
			}

			// show progress indicator if necessary
			if (vert >= nextProgress)
			{
				chomp::homology::scon << std::setw (3) <<
					(percentage + 75) << "%\b\b\b\b";
				++ percentage;
				nextProgress = (percentage * compSize) / 25;
			}
		}

		// make a correction to the total minimum if necessary
		if (!comp || (minCompWeight < minWeight))
			minWeight = minCompWeight;
	}

	// release allocated memory
	delete [] components;
	delete [] numbers;
	for (int_t i = 0; i <= maxCompSize; ++ i)
	{
		finite [i]. free ();
		delete [] (F [i]);
	}
	delete [] finite;
	delete [] F;

	// return the computed minimum
	return minWeight;
} /* minMeanCycleWeight */


// --------------------------------------------------

/// A version of Karp algorithm modified for the purpose
/// of interval arithmetic to provide the correct lower bound
/// for the minimum mean cycle weight in a graph.
/// This specialization is necessary, because of the subtraction
/// operation used in Karp's algorithm. Therefore, upper and lower
/// bounds for the minimum path progression weights must be computed
/// independently.
/// The intervals are compared by comparing their lower bounds only.
template <class wType, class roundType>
wType minMeanCycleWeight (const diGraph<wType> &g,
	const roundType &rounding, diGraph<wType> *transposed)
{
	// find the strongly connected components of the graph
	chomp::homology::multitable<int_t> compVertices, compEnds;
	bool copyweights = !!transposed;
	int_t countSCC = SCC (g, compVertices, compEnds,
		static_cast<diGraph<wType> *> (0), transposed, copyweights);
	if (countSCC <= 0)
		return 0;

	// compute the maximum size of each strongly connected component
	int_t maxCompSize = compEnds [0];
	for (int_t comp = 1; comp < countSCC; ++ comp)
	{
		int_t compSize = compEnds [comp] - compEnds [comp - 1];
		if (maxCompSize < compSize)
			maxCompSize = compSize;
	}

	// allocate arrays for weights and bit fields
	wType **FUpper = new wType * [maxCompSize + 1];
	wType **FLower = new wType * [maxCompSize + 1];
	chomp::homology::BitField *finite =
		new chomp::homology::BitField [maxCompSize + 1];
	for (int_t i = 0; i <= maxCompSize; ++ i)
	{
		FUpper [i] = new wType [maxCompSize];
		FLower [i] = new wType [maxCompSize];
		finite [i]. allocate (maxCompSize);
	}

	// compute the numbers of vertices in each component
	int_t nVertices (g. countVertices ());
	int_t *numbers = new int_t [nVertices];
	int_t *components = new int_t [nVertices];
	for (int_t i = 0; i < nVertices; ++ i)
		components [i] = -1;
	int_t offset = 0;
	for (int_t comp = 0; comp < countSCC; ++ comp)
	{
		int_t maxOffset = compEnds [comp];
		int_t pos = offset;
		for (; pos < maxOffset; ++ pos)
		{
			numbers [compVertices [pos]] = pos - offset;
			components [compVertices [pos]] = comp;
		}
		offset = pos;
	}

	// compute the minimum mean cycle weight for each component
	wType minWeight (0);
	for (int_t comp = 0; comp < countSCC; ++ comp)
	{
		int_t offset = comp ? compEnds [comp - 1] :
			static_cast<int_t> (0);
		int_t compSize = compEnds [comp] - offset;
		for (int_t i = 0; i <= compSize; ++ i)
			finite [i]. clearall (compSize);
		FUpper [0] [0] = 0;
		FLower [0] [0] = 0;
		finite [0]. set (0);

		// when to show the next progress indicator?
		int_t nextProgress = (compSize < 1000) ? (compSize + 1) : 0;
		short percentage = 0;

		// compute path progressions of given length
		for (int_t len = 1; len <= compSize; ++ len)
		{
			// process source vertices
			for (int_t i = 0; i < compSize; ++ i)
			{
				if (!finite [len - 1]. test (i))
					continue;
				wType prevUpper = FUpper [len - 1] [i];
				wType prevLower = FLower [len - 1] [i];
				int_t source = compVertices [offset + i];

				// process target vertices (and edges)
				int_t nEdges = g. countEdges (source);
				for (int edge = 0; edge < nEdges; ++ edge)
				{
					int_t target =
						g. getEdge (source, edge);
					if (components [target] != comp)
						continue;
					int_t j = numbers [target];
					wType newUpper = rounding. add_up
						(prevUpper,
						g. getWeight (source, edge));
					wType newLower = rounding. add_down
						(prevLower,
						g. getWeight (source, edge));
					if (!finite [len]. test (j))
					{
						finite [len]. set (j);
						FUpper [len] [j] = newUpper;
						FLower [len] [j] = newLower;
					}
					else
					{
						wType &curUpper =
							FUpper [len] [j];
						if (newUpper < curUpper)
							curUpper = newUpper;
						wType &curLower =
							FLower [len] [j];
						if (newLower < curLower)
							curLower = newLower;
					}
				}
			}

			// show progress indicator if necessary
			if (len >= nextProgress)
			{
				chomp::homology::scon << std::setw (3) <<
					percentage << "%\b\b\b\b";
				++ percentage;
				nextProgress = (percentage * compSize) / 75;
			}
		}

		// compute the minimum mean cycle weight for this component
		nextProgress = (compSize < 1000) ? (compSize + 1) : 0;
		percentage = 0;
		wType minCompWeight (0);
		bool firstMin = true;
		for (int_t vert = 0; vert < compSize; ++ vert)
		{
			if (!finite [compSize]. test (vert))
				continue;
			bool firstAverage = true;
			wType maxAverage = 0;
			for (int_t first = 0; first < compSize; ++ first)
			{
				if (!finite [first]. test (vert))
					continue;
				const wType diff = rounding. sub_down
					(FLower [compSize] [vert],
					FUpper [first] [vert]);
				wType average = rounding. div_down
					(diff, compSize - first);
				if (firstAverage)
				{
					maxAverage = average;
					firstAverage = false;
				}
				else if (maxAverage < average)
					maxAverage = average;
			}
			if (firstMin || (maxAverage < minCompWeight))
			{
				if (firstAverage)
					throw "Bug in Karp's algorithm";
				minCompWeight = maxAverage;
				firstMin = false;
			}

			// show progress indicator if necessary
			if (vert >= nextProgress)
			{
				chomp::homology::scon << std::setw (3) <<
					(percentage + 75) << "%\b\b\b\b";
				++ percentage;
				nextProgress = (percentage * compSize) / 25;
			}
		}

		// make a correction to the total minimum if necessary
		if (!comp || (minCompWeight < minWeight))
			minWeight = minCompWeight;
	}

	// release allocated memory
	delete [] components;
	delete [] numbers;
	for (int_t i = 0; i <= maxCompSize; ++ i)
	{
		finite [i]. free ();
		delete [] (FUpper [i]);
		delete [] (FLower [i]);
	}
	delete [] finite;
	delete [] FUpper;
	delete [] FLower;

	// return the computed minimum
	return minWeight;
} /* minMeanCycleWeight */


// --------------------------------------------------

/// A rigorous numerics version of Karp algorithm
/// modified in such a way that the memory usage is minimized,
/// at the cost of slower execution (up to twice slower).
/// Note that a memory conservative version of the algorithm
/// that uses non-rigorous numerics is not programmed yet.
template <class wType, class roundType>
wType minMeanCycleWeightMem (const diGraph<wType> &g,
	const roundType &rounding, diGraph<wType> *transposed)
{
// NOTE: Support for CHECK_ROUNDING_WIDTH is currently in preparation.

	// find the strongly connected components of the graph
	chomp::homology::multitable<int_t> compVertices, compEnds;
	bool copyweights = !!transposed;
	int_t countSCC = SCC (g, compVertices, compEnds,
		static_cast<diGraph<wType> *> (0), transposed, copyweights);
	if (countSCC <= 0)
		return 0;

	// compute the maximum size of each strongly connected component
	int_t maxCompSize = compEnds [0];
	for (int_t comp = 1; comp < countSCC; ++ comp)
	{
		int_t compSize = compEnds [comp] - compEnds [comp - 1];
		if (maxCompSize < compSize)
			maxCompSize = compSize;
	}

	// allocate arrays for weights and bit fields;
	// table 0 for F [0], tables 1 and 2 for F [k - 1] and F [k],
	// table 3 for F [n], table 4 for max of the quotients
	const int nTables (5);
	wType **FUpper = new wType * [nTables];
	wType **FLower = new wType * [nTables];
#ifdef CHECK_ROUNDING_WIDTH
	wType **FUpperE = new wType * [nTables];
	wType **FLowerE = new wType * [nTables];
#endif
	chomp::homology::BitField *finite =
		new chomp::homology::BitField [nTables];
	for (int_t i = 0; i < nTables; ++ i)
	{
		FUpper [i] = new wType [maxCompSize];
		FLower [i] = new wType [maxCompSize];
#ifdef CHECK_ROUNDING_WIDTH
		FUpperE [i] = new wType [maxCompSize];
		FLowerE [i] = new wType [maxCompSize];
#endif
		finite [i]. allocate (maxCompSize);
	}

	// compute the numbers of vertices in each component
	int_t nVertices (g. countVertices ());
	int_t *numbers = new int_t [nVertices];
	int_t *components = new int_t [nVertices];
	for (int_t i = 0; i < nVertices; ++ i)
		components [i] = -1;
	int_t offset = 0;
	for (int_t comp = 0; comp < countSCC; ++ comp)
	{
		int_t maxOffset = compEnds [comp];
		int_t pos = offset;
		for (; pos < maxOffset; ++ pos)
		{
			numbers [compVertices [pos]] = pos - offset;
			components [compVertices [pos]] = comp;
		}
		offset = pos;
	}

	// compute the minimum mean cycle weight for each component
	wType minWeight (0);
#ifdef CHECK_ROUNDING_WIDTH
	wType minWeightE (0);
#endif
	for (int_t comp = 0; comp < countSCC; ++ comp)
	{
		int_t offset = comp ? compEnds [comp - 1] :
			static_cast<int_t> (0);
		int_t compSize = compEnds [comp] - offset;
		finite [0]. clearall (compSize);
		finite [4]. clearall (compSize);
		FUpper [0] [0] = 0;
		FLower [0] [0] = 0;
#ifdef CHECK_ROUNDING_WIDTH
		FUpperE [0] [0] = 0;
		FLowerE [0] [0] = 0;
#endif
		finite [0]. set (0);

		// when to show the next progress indicator?
		int_t nextProgress = (compSize < 1000) ? (compSize + 1) : 0;
		short percentage = 0;

		// Pass 1: compute path progressions of maximal length
		const int_t maxLength1 (compSize + 1);
		for (int_t len = 1; len < maxLength1; ++ len)
		{
			// determine tables to use and clear the current one
			int_t prevIndex ((len == 1) ?
				0 : (((len - 1) & 1) + 1));
			int_t curIndex ((len == compSize) ?
				3 : ((len & 1) + 1));
			finite [curIndex]. clearall (compSize);

			// process all the edges starting at vertices
			// to which finite paths are already known
			for (int_t i = finite [prevIndex].
				find (0, compSize);
				(i >= 0) && (i < compSize);
				i = finite [prevIndex]. find
				(i + 1, compSize))
		//	for (int_t i = 0; i < compSize; ++ i)
			{
			//	if (!finite [prevIndex]. test (i))
			//		throw "Programming bug: infinite.";
				int_t source = compVertices [offset + i];

				// process target vertices (and edges)
				int_t nEdges = g. countEdges (source);
				for (int edge = 0; edge < nEdges; ++ edge)
				{
					int_t target =
						g. getEdge (source, edge);
					if (components [target] != comp)
						continue;
					int_t j = numbers [target];
					wType newLower = rounding. add_down
						(FLower [prevIndex] [i],
						g. getWeight (source, edge));
#ifdef CHECK_ROUNDING_WIDTH
					wType newLowerE = rounding. add_up
						(FLowerE [prevIndex] [i],
						g. getWeight (source, edge));
#endif

					// if this is the first time that
					// a path of this length
					// to the vertex was encountered
					if (!finite [curIndex]. test (j))
					{
						finite [curIndex]. set (j);
						FLower [curIndex] [j] =
							newLower;
#ifdef CHECK_ROUNDING_WIDTH
						FLowerE [curIndex] [j] =
							newLowerE;
#endif
					}
					// if a better path of this length
					// to the vertex was found
					else if (newLower <
						FLower [curIndex] [j])
					{
						FLower [curIndex] [j] =
							newLower;
#ifdef CHECK_ROUNDING_WIDTH
						FLowerE [curIndex] [j] =
							newLowerE;
#endif
					}
				}
			}

			// show progress indicator if necessary
			if (len >= nextProgress)
			{
				chomp::homology::scon << std::setw (3) <<
					percentage << "%\b\b\b\b";
				++ percentage;
				nextProgress = (percentage * compSize) / 50;
			}
		}

		// set up the mean average at level 0
		finite [4]. set (0);
		FLower [4] [0] = rounding. div_down (FLower [3] [0],
			compSize);
#ifdef CHECK_ROUNDING_WIDTH
		FLowerE [4] [0] = rounding. div_up (FLowerE [3] [0],
			compSize);
#endif

		// Pass 2: compute path progressions of all the lengths and
		// update the max values of the quotients for each vertex
		nextProgress = (compSize < 1000) ? (compSize + 1) : 0;
		percentage = 0;
		const int_t maxLength2 (compSize);
		for (int_t len = 1; len < maxLength2; ++ len)
		{
			// determine tables to use and clear the current one
			int_t prevIndex ((len == 1) ?
				0 : (((len - 1) & 1) + 1));
			int_t curIndex ((len & 1) + 1);
			finite [curIndex]. clearall (compSize);

			// process all the edges starting at vertices
			// to which finite paths are already known
			for (int_t i = finite [prevIndex].
				find (0, compSize);
				(i >= 0) && (i < compSize);
				i = finite [prevIndex]. find
				(i + 1, compSize))
		//	for (int_t i = 0; i < compSize; ++ i)
			{
				if (!finite [prevIndex]. test (i))
					throw "Programming bug: infinite.";
				int_t source = compVertices [offset + i];

				// process target vertices (and edges)
				int_t nEdges = g. countEdges (source);
				for (int edge = 0; edge < nEdges; ++ edge)
				{
					int_t target =
						g. getEdge (source, edge);
					if (components [target] != comp)
						continue;
					int_t j = numbers [target];
					wType newUpper = rounding. add_up
						(FUpper [prevIndex] [i],
						g. getWeight (source, edge));
#ifdef CHECK_ROUNDING_WIDTH
					wType newUpperE = rounding. add_down
						(FUpperE [prevIndex] [i],
						g. getWeight (source, edge));
#endif
					bool checkAverage = false;

					// if this is the first time that
					// a path of this length
					// to the vertex was encountered
					if (!finite [curIndex]. test (j))
					{
						finite [curIndex]. set (j);
						FUpper [curIndex] [j] =
							newUpper;
#ifdef CHECK_ROUNDING_WIDTH
						FUpperE [curIndex] [j] =
							newUpperE;
#endif
						checkAverage = true;
					}
					// if a better path of this length
					// to the vertex was found
					else if (newUpper <
						FUpper [curIndex] [j])
					{
						FUpper [curIndex] [j] =
							newUpper;
#ifdef CHECK_ROUNDING_WIDTH
						FUpperE [curIndex] [j] =
							newUpperE;
#endif
						checkAverage = true;
					}

					// update the max average weight
					if (!checkAverage)
						continue;
					const wType diff = rounding. sub_down
						(FLower [3] [j],
						FUpper [curIndex] [j]);
#ifdef CHECK_ROUNDING_WIDTH
					const wType diffE = rounding. sub_up
						(FLowerE [3] [j],
						FUpperE [curIndex] [j]);
#endif
					const wType average = rounding.
						div_down (diff,
						compSize - len);
#ifdef CHECK_ROUNDING_WIDTH
					const wType averageE = rounding.
						div_up (diffE,
						compSize - len);
#endif
					if (!finite [4]. test (j))
					{
						finite [4]. set (j);
						FLower [4] [j] = average;
#ifdef CHECK_ROUNDING_WIDTH
						FLowerE [4] [j] = averageE;
#endif
					}
					else if (FLower [4] [j] < average)
					{
						FLower [4] [j] = average;
#ifdef CHECK_ROUNDING_WIDTH
						FLowerE [4] [j] = averageE;
#endif
					}
				}
			}

			// show progress indicator if necessary
			if (len >= nextProgress)
			{
				chomp::homology::scon << std::setw (3) <<
					(percentage + 50) << "%\b\b\b\b";
				++ percentage;
				nextProgress = (percentage * compSize) / 50;
			}
		}

		// compute the minimum mean cycle weight for this component
		wType minCompWeight (0);
#ifdef CHECK_ROUNDING_WIDTH
		wType minCompWeightE (0);
#endif
		for (int_t vert = 0; vert < compSize; ++ vert)
		{
		//	if (!finite [4]. test (vert))
		//		throw "Programming bug: Undefined average.";
			if (!vert || (FLower [4] [vert] < minCompWeight))
			{
				minCompWeight = FLower [4] [vert];
#ifdef CHECK_ROUNDING_WIDTH
				minCompWeightE = FLowerE [4] [vert];
#endif
			}
		}

		// make a correction to the total minimum if necessary
		if (!comp || (minCompWeight < minWeight))
		{
			minWeight = minCompWeight;
#ifdef CHECK_ROUNDING_WIDTH
			minWeightE = minCompWeightE;
#endif
		}
	}

	// release allocated memory
	delete [] components;
	delete [] numbers;
	for (int_t i = 0; i < nTables; ++ i)
	{
		finite [i]. free ();
		delete [] (FUpper [i]);
		delete [] (FLower [i]);
#ifdef CHECK_ROUNDING_WIDTH
		delete [] (FUpperE [i]);
		delete [] (FLowerE [i]);
#endif
	}
	delete [] finite;
	delete [] FUpper;
	delete [] FLower;
#ifdef CHECK_ROUNDING_WIDTH
	delete [] FUpperE;
	delete [] FLowerE;
#endif

#ifdef CHECK_ROUNDING_WIDTH
	// show the loss of precision caused by rounding
	chomp::homology::sbug << "\nLoss of precision info: [" <<
		minWeight << ", " << minWeightE <<
		"], width: " << (minWeightE - minWeight) << ".\n";
#endif

	// return the computed minimum
	return minWeight;
} /* minMeanCycleWeightMem */


} // namespace cymealg

#endif // _CYMEALG_CYCLEMEAN_H_

/// @}

