/// @addtogroup cymealg
/// @{

/////////////////////////////////////////////////////////////////////////////
///
/// @file
///
/// This header file contains several procedures directly related
/// to the DFS algorithm.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2020 by Pawel Pilarczyk.
//
// This file is part of my research software. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started in January 2006. Last revision: August 23, 2019.


#ifndef _CYMEALG_DFS_H_
#define _CYMEALG_DFS_H_

// include selected header files from the CHomP library
#include "chomp/system/config.h"

namespace cymealg {

/// The recurrent procedure for DFSfinishTime.
template <class Graph, class Table>
inline void DFSfinishTimeRecurrent (const Graph &g, Table &tab,
	int_t vertex, int_t &counter)
{
	g. DFSfinishTimeRecurrent (tab, vertex, counter);
	return;
} /* DFSfinishTimeRecurrent */

/// A stack version of the recurrent procedure for DFSfinishTime.
template <class Graph, class Table>
inline void DFSfinishTimeStack (const Graph &g, Table &tab,
	int_t vertex, int_t &counter)
{
	g. DFSfinishTimeStack (tab, vertex, counter);
	return;
} /* DFSfinishTimeStack */

/// Computes the DFS finishing time for each vertex.
/// Note: The time begins with 1, not with 0.
template <class Graph, class Table>
inline void DFSfinishTime (const Graph &g, Table &tab)
{
	g. DFSfinishTime (tab);
	return;
} /* DFSfinishTime */

/// The recurrent procedure for DFSforest.
/// Returns true iff there is a loop within the tree found.
template <class Graph, class Table1, class Table2>
inline bool DFSforestRecurrent (const Graph &g, Table1 &tab, Table1 &ntab,
	int_t vertex, int_t treeNumber, int_t countTrees,
	Table2 &compVertices, int_t &curVertex,
	Graph *sccGraph, int_t *sccEdgeAdded)
{
	return g. DFSforestRecurrent (tab, ntab, vertex,
		treeNumber, countTrees, compVertices,
		curVertex, sccGraph, sccEdgeAdded);
} /* DFSforestRecurrent */

/// A stack version of the recurrent procedure for DFSforest.
template <class Graph, class Table1, class Table2>
inline bool DFSforestStack (const Graph &g, Table1 &tab, Table1 &ntab,
	int_t vertex, int_t treeNumber, int_t countTrees,
	Table2 &compVertices, int_t &curVertex,
	Graph *sccGraph, int_t *sccEdgeAdded)
{
	return g. DFSforestStack (g, tab, ntab, vertex,
		treeNumber, countTrees, compVertices,
		curVertex, sccGraph, sccEdgeAdded);
} /* DFSforestStack */

/// Computes the DFS forest.
/// Considers the vertices in the given order.
/// Saves the numbers of vertices of each tree in 'compVertices',
/// and keeps the one-beyond-the-end offsets of the trees in the
/// table 'compEnds'. Records the connections between the trees
/// in 'scc' (which must be empty when this function is called).
/// If requested, only those single-vertex trees are counted
/// which have an edge that loops back to themselves.
/// Returns the number of trees in the computed forest.
template <class Graph, class Table1, class Table2, class Table3>
inline int_t DFSforest (const Graph &g, const Table1 &ordered,
	Table2 &compVertices, Table3 &compEnds, bool nontrivial = false,
	Graph *sccGraph = 0)
{
	return g. DFSforest (ordered, compVertices, compEnds, nontrivial,
		sccGraph);
} /* DFSforest */

/// The recurrent procedure for DFScolor.
template <class Graph, class Table, class Color>
inline void DFScolorRecurrent (const Graph &g, Table &tab,
	const Color &color, int_t vertex = 0)
{
	g. DFScolorRecurrent (tab, color, vertex);
	return;
} /* DFScolorRecurrent */

/// A stack version of the recurrent procedure for DFScolor.
template <class Graph, class Table, class Color>
inline void DFScolorStack (const Graph &g, Table &tab, const Color &color,
	int_t vertex = 0)
{
	g. DFScolorStack (tab, color, vertex);
	return;
} /* DFScolorStack */

/// Marks each vertex visited by DFS with the given color,
/// starting with the given vertex. Runs for one component only.
/// The initial color in 'tab' must be different than the given one.
template <class Graph, class Table, class Color>
inline void DFScolor (const Graph &g, Table &tab, const Color &color,
	int_t vertex = 0)
{
	g. DFScolor (tab, color, vertex);
	return;
} /* DFScolor */


} // namespace cymealg

#endif // _CYMEALG_DFS_H_

/// @}

