/// @addtogroup cymealg
/// @{

/////////////////////////////////////////////////////////////////////////////
///
/// @file
///
/// This header file contains the definition of a few functions
/// for interfacing with directed graphs using tables.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2020 by Pawel Pilarczyk.
//
// This file is part of my research software. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started in January 2006. Last revision: August 23, 2019.


#ifndef _CYMEALG_DIGRAPHTAB_H_
#define _CYMEALG_DIGRAPHTAB_H_

// include selected CHomP header files
#include "chomp/system/config.h"

namespace cymealg {

/// Sets the weights of all the edges at a time.
/// The weights are pulled from the table with the [] operator.
template <class Graph, class Table>
inline void setWeights (Graph &g, const Table &tab)
{
	int_t nEdges (g. countEdges ());
	for (int_t i = 0; i < nEdges; ++ i)
		g. setWeight (i, tab [i]);
	return;
} /* setWeights */

/// Gets the weights of all the edges at a time.
/// The weights are put into the table with the [] operator.
template <class Graph, class Table>
inline void getWeights (const Graph &g, Table &tab)
{
	int_t nEdges (g. countEdges ());
	for (int_t i = 0; i < nEdges; ++ i)
		tab [i] = g. getWeight (i);
	return;
} /* getWeights */

/// Fills out a table that represents all the edges of the graph.
/// The indices of a starting and ending vertex of the n-th edge
/// are written to "tab [n] [0]" and "tab [n] [1]", respectively.
/// The weights are ignored; use "getWeights" to store them separately.
template <class Graph, class Table>
inline void writeEdges (const Graph &g, Table &tab)
{
	int_t nVertices (g. countVertices ());
	int_t curEdge (0);
	for (int_t curVertex = 0; curVertex < nVertices; ++ curVertex)
	{
		int_t nEdges = g. countEdges (curVertex);
		for (int_t i = 0; i < nEdges; ++ i)
		{
			tab [curEdge] [0] = curVertex;
			tab [curEdge] [1] = g. getEdge (curVertex, i);
			++ curEdge;
		}
	}
	return;
} /* diGraph::writeEdges */


} // namespace cymealg

#endif // _CYMEALG_DIGRAPHTAB_H_

/// @}

