/// @addtogroup cymealg
/// @{

/////////////////////////////////////////////////////////////////////////////
///
/// @file
///
/// This header file contains a function for the computation of the minimum
/// mean path weight. The function is based on Karp's algorithm
/// for computing the minimum cycle mean weight.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2020 by Pawel Pilarczyk.
//
// This file is part of my research software. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started in January 2006. Last revision: August 23, 2019.


#ifndef _CYMEALG_PATHMEAN_H_
#define _CYMEALG_PATHMEAN_H_

// include selected header files from the CHomP library
#include "chomp/system/config.h"

// include selected CyMeAlg header files
#include "cymealg/dummyrnd.h"

namespace cymealg {

/// Runs an algorithm based on Karp's idea to compute the minimum
/// mean path weight for paths starting at any of the given
/// n vertices and of length not exceeding the number of vertices
/// in the graph.
/// Returns 0 if no path starts at any of the vertices.
template <class wType, class arrayType, class roundType>
wType minMeanPathWeight (const diGraph<wType> &g,
	const roundType &rounding, const arrayType &starting, int_t n)
{
	// allocate arrays for weights and bit fields
	const int nIndices = 2;
	wType **F = new wType * [nIndices];
	chomp::homology::BitField *finite =
		new chomp::homology::BitField [nIndices];
	int_t nVertices (g. countVertices ());
	for (int i = 0; i < nIndices; ++ i)
	{
		F [i] = new wType [nVertices];
		finite [i]. allocate (nVertices);
	}

	// set the zero path lengths from the initial vertices
	for (int_t i = 0; i < n; ++ i)
	{
		int_t v = starting [i];
		if ((v < 0) || (v >= nVertices))
			throw "Starting vertex out of range.";
		F [0] [v] = 0;
		finite [0]. set (v);
	}

	// compute path progressions of given length and average weights
	wType minWeight (0);
	bool firstWeight = true;
	int_t maxLength (nVertices + 1);
	for (int_t len = 1; len < maxLength; ++ len)
	{
		// determine the indices for previous and current paths
		int_t prevIndex = (len - 1) & 1;
		int_t curIndex = len & 1;
		finite [curIndex]. clearall (nVertices);

		// process source vertices
		for (int_t source = 0; source < nVertices; ++ source)
		{
			if (!finite [prevIndex]. test (source))
				continue;
			wType prevWeight = F [prevIndex] [source];

			// process target vertices (and edges)
			int_t nEdges = g. countEdges (source);
			for (int edge = 0; edge < nEdges; ++ edge)
			{
				int_t target =
					g. getEdge (source, edge);
				wType newWeight = rounding. add_down
					(prevWeight,
					g. getWeight (source, edge));
				if (!finite [curIndex]. test (target))
				{
					finite [curIndex]. set (target);
					F [curIndex] [target] = newWeight;
				}
				else if (newWeight < F [curIndex] [target])
					F [curIndex] [target] = newWeight;
			}
		}

		// update the minimum mean path weight
		for (int_t vert = 0; vert < nVertices; ++ vert)
		{
			if (!finite [curIndex]. test (vert))
				continue;
			wType average = rounding. div_down
				(F [curIndex] [vert], len);
			if (firstWeight)
			{
				minWeight = average;
				firstWeight = false;
			}
			else if (average < minWeight)
				minWeight = average;
		}
	}

	// release allocated memory
	for (int i = 0; i < nIndices; ++ i)
	{
		finite [i]. free ();
		delete [] (F [i]);
	}
	delete [] finite;
	delete [] F;

	// return the computed minimum
	return minWeight;
} /* minMeanPathWeight */

// --------------------------------------------------

/// The above algorithm without rounding control.
template <class wType, class arrayType>
wType minMeanPathWeight (const diGraph<wType> &g,
	const arrayType &starting, int_t n)
{
	const dummyRounding<wType> rounding = dummyRounding<wType> ();
	return minMeanPathWeight (g, rounding, starting, n);
} /* minMeanPathWeight */


} // namespace cymealg

#endif // _CYMEALG_PATHMEAN_H_

/// @}

