/// @addtogroup cymealg
/// @{

/////////////////////////////////////////////////////////////////////////////
///
/// @file
///
/// This header file contains functions for reading and writing digraphs
/// from and to human-readable text streams.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2020 by Pawel Pilarczyk.
//
// This file is part of my research software. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started in January 2006. Last revision: August 23, 2019.


#ifndef _CYMEALG_READWRITE_H_
#define _CYMEALG_READWRITE_H_

// include selected header files from the CHomP library
#include "chomp/system/config.h"
#include "chomp/system/textfile.h"

namespace cymealg {

/// Reads a graph in the text mode from an input stream.
/// The graph must be initially empty, or otherwise the graph
/// read from the input stream is appended to the existing graph.
template <class inType, class Graph>
inline inType &read (inType &in, Graph &g)
{
	chomp::homology::ignorecomments (in);

	// the number of vertices in the input graph (optional);
	// necessary if there are no outgoing edges from the last vertex
	int_t nVertices (-1);

	// read all the edges in the graph and add them
	int_t curVertex (g. countVertices () - 1);
	int_t curEdge (0);
	while (!in. eof ())
	{
		// read the source vertex number
		int_t vertex (-1);
		in >> vertex;
		chomp::homology::ignorecomments (in);

		// if this was the number of vertices then remember it
		if ((nVertices < 0) && (in. peek () != '-'))
		{
			nVertices = vertex;
			chomp::homology::ignoreline (in);
			chomp::homology::ignorecomments (in);
			continue;
		}

		// read the arrow "->"
		if (in. peek () == '-')
			in. get ();
		if (in. peek () == '>')
			in. get ();
		chomp::homology::ignorecomments (in);

		// read the target vertex number of the edge
		int_t target (-1);
		in >> target;
		chomp::homology::ignorecomments (in);

		// read the edge weight if it is provided
		typename Graph::weight_type weight;
		bool weightRead (false);
		if (in. peek () == '[')
		{
			in. get ();
			in >> weight;
			weightRead = true;
			if (in. peek () == ']')
				in. get ();
		}
		chomp::homology::ignorecomments (in);

		// add as many vertices to the graph as necessary
		if (vertex < curVertex)
			throw "Wrong order of vertices in the input stream.";
		while (vertex > curVertex)
		{
			g. addVertex ();
			++ curVertex;
			curEdge = 0;
		}

		// add the edge to the graph
		g. addEdge (target);

		// add the weight of the edge if it was read
		if (weightRead)
			g. setWeight (curVertex, curEdge, weight);

		// increase the counter of edges of the current vertex
		++ curEdge;
	}

	// add the remaining vertices to the graph
	while (g. countVertices () < nVertices)
		g. addVertex ();

	return in;
} /* read */

// --------------------------------------------------

/// Reads a graph in the text mode from an input stream.
template <class wType>
inline std::istream &operator >> (std::istream &in, diGraph<wType> &g)
{
	return read (in, g);
} /* operator >> */

// --------------------------------------------------

/// Outputs the graph to a text stream in a human-readable format.
template <class outType, class Graph>
inline outType &write (outType &out, const Graph &g, bool writeWeights)
{
	int_t nVertices (g. countVertices ());
	out << "; Directed graph.\n";
	out << nVertices << " vertices.\n";
	for (int_t i = 0; i < nVertices; ++ i)
	{
		int_t nEdges (g. countEdges (i));
		for (int_t j = 0; j < nEdges; ++ j)
		{
			out << i << " -> " << g. getEdge (i, j);
			if (writeWeights)
				out << " [" << g. getWeight (i, j) << "]\n";
			else
				out << "\n";
		}
	}
	out << "; This is the end of the graph.\n";
	return out;
} /* write */

// --------------------------------------------------

/// Writes a graph in the text mode to an output stream.
template <class wType>
inline std::ostream &operator << (std::ostream &out, const diGraph<wType> &g)
{
	return write (out, g, true);
} /* operator << */


} // namespace cymealg

#endif // _CYMEALG_READWRITE_H_

/// @}

