/// @addtogroup cymealg
/// @{

/////////////////////////////////////////////////////////////////////////////
///
/// @file
///
/// This header file contains the implementation of two algorithms
/// for the computation of strongly connected path components of a digraph.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2020 by Pawel Pilarczyk.
//
// This file is part of my research software. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started in January 2006. Last revision: August 23, 2019.


#ifndef _CYMEALG_SCC_H_
#define _CYMEALG_SCC_H_

// include selected standard library header files
#include <stack>
#include <vector>

// include selected header files from the CHomP library
#include "chomp/system/config.h"
#include "chomp/struct/bitfield.h"

// include selected CyMeAlg header files
#include "cymealg/transpose.h"
#include "cymealg/dfs.h"

namespace cymealg {

/// Computes strongly connected path components of the graph 'g'. Creates
/// the graph 'scc' in which each vertex corresponds to one component.
/// The graph 'scc' given as an argument must be initially empty.
/// The table 'compVertices' is filled with the numbers of vertices in 'g'
/// which form the components, and the indices that end the listing
/// for each component are stored in the table 'compEnds'.
/// Returns the number of strongly connected components found.
template <class wType, class Table1, class Table2>
inline int_t SCC (const diGraph<wType> &g, Table1 &compVertices,
	Table2 &compEnds, diGraph<wType> *scc = 0,
	diGraph<wType> *transposed = 0, bool copyweights = false)
{
	// prepare two tables
	int_t nVert = g. countVertices ();
	int_t *ordered = new int_t [nVert];
	int_t *tab = new int_t [nVert];

	// compute the list of vertices in the descending finishing time
	g. DFSfinishTime (tab);
	for (int_t i = 0; i < nVert; ++ i)
		ordered [nVert - tab [i]] = i;
	delete [] tab;

	// create the transposed graph
	diGraph<wType> gT;
	if (!transposed)
		transposed = &gT;
	transpose (g, *transposed, copyweights);

	// extract the DFS forest of gT in the given order of vertices
	int_t n = DFSforest (*transposed, ordered, compVertices, compEnds,
		true, scc);

	// cleanup memory and return the number of components
	delete [] ordered;
	return n;
} /* SCC */

// --------------------------------------------------

/// Computes strongly connected components of the graph 'g'
/// using Tarjan's algorithm (as described in the Wikipedia).
/// Tha advantage of this approach over the one described in Cormen's
/// textbook is that the transposed graph need not be computed.
/// However, this algorithm might be slightly slower than the other one.
/// The table 'compVertices' is filled with the numbers of vertices in 'g'
/// which form the components, and the indices that end the listing
/// for each component are stored in the table 'compEnds'.
/// Returns the number of strongly connected components found.
template <class wType, class Table1, class Table2>
inline int_t SCC_Tarjan (const diGraph<wType> &g, Table1 &compVertices,
	Table2 &compEnds)
{
	// return the obvious result if the graph is empty
	int_t nVertices = g. countVertices ();
	if (!nVertices)
		return 0;

	// prepare an array of discovery times for all the vertices
	// (zero == not yet discovered)
	std::vector<int_t> dfsIndex (nVertices, 0);

	// prepare an array of the minimal index of a node reachable
	// from each of the vertices
	std::vector<int_t> lowLink (nVertices, 0);

	// prepare an empty stack of nodes
	std::stack<int_t> s_nodes;

	// prepare an array of bits indicating whether the vertices are
	// in the stack of nodes or not
	chomp::homology::BitField inTheStack;
	inTheStack. allocate (nVertices);
	inTheStack. clearall (nVertices);

	// prepare the number of strongly connected components
	int_t nComponents = 0;

	// prepare the current position in the array 'compVertices'
	int_t posVertices = 0;

	// remember the next vertex number in the graph to scan
	// whether this vertex has already been visited or not
	int_t vertexToScan = 0;

	// prepare a variable for storing the discovery time in the DFS
	int_t discoveryTime = 0;

	// prepare stacks for the DFS recursion
	std::stack<int_t> s_vertex;
	std::stack<int_t> s_edge;
	std::stack<int_t> s_maxedge;

	// initialize the number of the currently processed vertex
	int_t vertex = -1;

	// initialize the range of edges to be visited
	int_t edge = 0;
	int_t maxedge = 0;

	while (1)
	{
		// return to the previous recursion level
		// if all the edges have been checked
		if (edge >= maxedge)
		{
			// extract a strongly connected component
			if ((vertex >= 0) && (lowLink [vertex] ==
				dfsIndex [vertex]))
			{
				int_t v = 0;
				do
				{
					v = s_nodes. top ();
					s_nodes. pop ();
					inTheStack. clear (v);
					compVertices [posVertices ++] = v;
				} while (v != vertex);
				compEnds [nComponents ++] = posVertices;
			}

			// if this is the top level of the recursion
			// then find another unvisited vertex or return
			if (s_vertex. empty ())
			{
				// find an unvisited vertex in the graph
				while ((vertexToScan < nVertices) &&
					(dfsIndex [vertexToScan] != 0))
				{
					++ vertexToScan;
				}

				// return the result if all visited
				if (vertexToScan == nVertices)
				{
					inTheStack. free ();
					return nComponents;
				}

				// set the new vertex
				vertex = vertexToScan ++;

				// mark the current vertex as visited
				// and initialize its low link
				dfsIndex [vertex] = ++ discoveryTime;
				lowLink [vertex] = discoveryTime;

				// push this vertex on the stack
				s_nodes. push (vertex);
				inTheStack. set (vertex);

				// determine the edges to be visited
				edge = 0;
				maxedge = g. countEdges (vertex);
			}

			// otherwise trace back to the previous level
			else
			{
				// remember the current low link index
				int_t lowLink2 = lowLink [vertex];

				// restore the variables
				vertex = s_vertex. top ();
				s_vertex. pop ();
				edge = s_edge. top ();
				s_edge. pop ();
				maxedge = s_maxedge. top ();
				s_maxedge. pop ();

				// update the current low link index
				if (lowLink [vertex] > lowLink2)
					lowLink [vertex] = lowLink2;
			}
		}

		// analyse the next edge coming out from the current vertex
		else
		{
			// determine the next vertex
			int_t next = g. getEdge (vertex, edge ++);

			// go to a deeper recursion level if unvisited
			if (dfsIndex [next] == 0)
			{
				// store the variables at the stacks
				s_vertex. push (vertex);
				s_edge. push (edge);
				s_maxedge. push (maxedge);

				// set the new vertex
				vertex = next;
			
				// mark the new vertex as visited
				dfsIndex [vertex] = ++ discoveryTime;
				lowLink [vertex] = discoveryTime;
			
				// push this vertex on the stack
				s_nodes. push (vertex);
				inTheStack. set (vertex);

				// determine the edges to be visited
				edge = 0;
				maxedge = g. countEdges (vertex);
			}

			// update the low link index if the vertex has been
			// visited and is currently in the stack of nodes
			else if (inTheStack. test (next))
			{
				if (lowLink [vertex] > dfsIndex [next])
					lowLink [vertex] = dfsIndex [next];
			}
		}
	}

	// finalize and return the number of strongly connected components
	inTheStack. free ();
	return nComponents;
} /* SCC_Tarjan */


} // namespace cymealg

#endif // _CYMEALG_SCC_H_

/// @}

