/// @addtogroup cymealg
/// @{

/////////////////////////////////////////////////////////////////////////////
///
/// @file
///
/// This header file contains implementation of a selection
/// of various graph algorithms.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2020 by Pawel Pilarczyk.
//
// This file is part of my research software. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started in January 2006. Last revision: August 23, 2019.


#ifndef _CYMEALG_VARIOUS_H_
#define _CYMEALG_VARIOUS_H_

// include selected header files from the CHomP library
#include "chomp/system/config.h"

namespace cymealg {

/// A helper function for "collapseVertices".
template <class wType>
inline int_t addRenumEdges (const diGraph<wType> &g, int_t vertex,
	const int_t *newNum, int_t cur, int_t *srcVert,
	diGraph<wType> &result)
{
	int_t nEdges = g. countEdges (vertex);
	// add all the edges that start at the component
	for (int_t edge = 0; edge < nEdges; ++ edge)
	{
		// determine the dest. vertex of the edge
		int_t dest = newNum [g. getEdge (vertex, edge)];
		// if this is an edge to self, then ignore it
		if (dest == cur)
			continue;
		// if the edge has already been added,
		// then skip it
		if (srcVert [dest] == cur)
			continue;
		// remember that the dest. vertex has an edge
		// pointing out from the current vertex
		srcVert [dest] = cur;
		// add the edge to the result graph
		result. addEdge (dest);
	}
	return 0;
} /* addRenumEdges */

/// Collapses disjoint subsets of vertices to single vertices and creates
/// a corresponding graph in which the first vertices come from
/// the collapsed ones. The result graph must be initially empty.
/// Saves translation table from old to new vertex numbers.
/// The table must have sufficient length.
template <class wType, class Table1, class Table2>
inline int_t collapseVertices (const diGraph<wType> &g,
	int_t compNum, Table1 &compVertices, Table2 &compEnds,
	diGraph<wType> &result, int_t *newNumbers = 0)
{
	// do nothing if the input graph is empty
	int_t nVert = g. countVertices ();
	if (!nVert)
		return 0;

	// compute the new numbers of vertices: newNum [i] is the
	// number of vertex 'i' from 'g' in the resulting graph
	int_t *newNum = newNumbers ? newNumbers : new int_t [nVert];
	for (int_t i = 0; i < nVert; ++ i)
		newNum [i] = -1;
	int_t cur = 0; // current vertex number in the new graph
	int_t pos = 0; // position in compVertices
	while (cur < compNum)
	{
		int_t posEnd = compEnds [cur];
		while (pos < posEnd)
			newNum [compVertices [pos ++]] = cur;
		++ cur;
	}
	for (int_t i = 0; i < nVert; ++ i)
	{
		if (newNum [i] < 0)
			newNum [i] = cur ++;
	}

	// prepare a table to mark the most recent source vertex for edges:
	// srcVert [j] contains i if the edge i -> j has just been added
	int_t newVert = nVert - (compNum ? compEnds [compNum - 1] :
		static_cast<int_t> (0)) + compNum;
	// debug:
	if (cur != newVert)
		throw "DEBUG: Wrong numbers.";
	int_t *srcVert = new int_t [newVert];
	for (int_t i = 0; i < newVert; ++ i)
		srcVert [i] = -1;

	// scan the input graph and create the resulting graph: begin with
	// the vertices in the collapsed groups
	cur = 0, pos = 0;
	while (cur < compNum)
	{
		// add a new vertex to the result graph
		result. addVertex ();
		// for all the vertices in this component...
		int_t posEnd = compEnds [cur];
		while (pos < posEnd)
		{
			// take the next vertex from the component
			int_t vertex = compVertices [pos ++];
			// add the appropriate edges to the result graph
			addRenumEdges (g, vertex, newNum, cur,
				srcVert, result);
		}
		++ cur;
	}
	// process the remaining vertices of the graph
	for (int_t vertex = 0; vertex < nVert; ++ vertex)
	{
		// debug
		if (newNum [vertex] > cur)
			throw "DEBUG: Wrong order.";
		// if the vertex has already been processed, skip it
		if (newNum [vertex] != cur)
			continue;
		// add the appropriate edges to the result graph
		result. addVertex ();
		addRenumEdges (g, vertex, newNum, cur, srcVert, result);
		++ cur;
	}

	// free memory and exit
	delete [] srcVert;
	if (!newNumbers)
		delete [] newNum;
	return 0;
} /* collapseVertices */

/// A helper function for "collapseVertices2".
template <class wType, class TabSets>
inline int_t addRenumEdges2 (const diGraph<wType> &g, int_t vertex,
	const int_t *newNum, TabSets &numSets,
	int_t cur, int_t *srcVert, diGraph<wType> &result)
{
	int_t nEdges = g. countEdges (vertex);

	// add all the edges that start at this vertex
	for (int_t edge = 0; edge < nEdges; ++ edge)
	{
		// determine the dest. vertex of the edge
		int_t destNumber = newNum [g. getEdge (vertex, edge)];

		// consider all the destination vertices
		int_t destSize = (destNumber < 0) ?
			numSets [-destNumber]. size () :
			static_cast<int_t> (1);
		for (int_t i = 0; i < destSize; ++ i)
		{
			// determine the consecutive destination vertex
			int_t dest = (destNumber < 0) ?
				numSets [-destNumber] [i] : destNumber;

			// if this is an edge to self, then ignore it
			if (dest == cur)
				continue;

			// if the edge has already been added,
			// then skip it
			if (srcVert [dest] == cur)
				continue;

			// add the edge to the result graph
			result. addEdge (dest);
		}
	}
	return 0;
} /* addRenumEdges2 */

/// Collapses (possibly non-disjoint) subsets of vertices
/// to single vertices and creates a corresponding graph
/// in which the first vertices come from the collapsed ones.
/// The result graph must be initially empty.
template <class wType, class Table1, class Table2>
inline int_t collapseVertices2 (const diGraph<wType> &g,
	int_t compNum, Table1 &compVertices, Table2 &compEnds,
	diGraph<wType> &result)
{
	// do nothing if the input graph is empty
	int_t nVert = g. countVertices ();
	if (!nVert)
		return 0;

	// compute the new numbers of vertices: newNum [i] is the
	// number of vertex 'i' from 'g' in the resulting graph,
	// unless negative; then it points to a set of numbers,
	// with -1 meaning "not defined, yet"
	int_t *newNum = new int_t [nVert];
	for (int_t i = 0; i < nVert; ++ i)
		newNum [i] = -1;

	// sets of numbers of vertices; the numbers refering to the sets
	// begin with 2, thus the first two sets are skipped
	chomp::homology::multitable<chomp::homology::hashedset<int_t> >
		numSets;
	// the number of the set waiting to be used next
	int_t numSetCur = 2;

	int_t cur = 0; // current vertex number in the new graph
	int_t pos = 0; // position in compVertices
	while (cur < compNum)
	{
		int_t posEnd = compEnds [cur];
		while (pos < posEnd)
		{
			int_t number = compVertices [pos ++];
			if (newNum [number] == -1)
				newNum [number] = cur;
			else if (newNum [number] < 0)
				numSets [-newNum [number]]. add (cur);
			else
			{
				numSets [numSetCur]. add (newNum [number]);
				numSets [numSetCur]. add (cur);
				newNum [number] = -(numSetCur ++);
			}
		}
		++ cur;
	}
	for (int_t i = 0; i < nVert; ++ i)
	{
		if (newNum [i] == -1)
			newNum [i] = cur ++;
	}

	// prepare a table to mark the most recent source vertex for edges:
	// srcVert [j] contains i if the edge i -> j has just been added
	int_t newVert = cur;
	int_t *srcVert = new int_t [newVert];
	for (int_t i = 0; i < newVert; ++ i)
		srcVert [i] = -1;

	// scan the input graph and create the resulting graph: begin with
	// the vertices in the collapsed groups
	cur = 0;
	pos = 0;
	while (cur < compNum)
	{
		// add a new vertex to the result graph
		result. addVertex ();

		// for all the vertices in this component...
		int_t posEnd = compEnds [cur];
		while (pos < posEnd)
		{
			// take the next vertex from the component
			int_t vertex = compVertices [pos ++];

			// add the appropriate edges to the result graph
			addRenumEdges2 (g, vertex, newNum, numSets, cur,
				srcVert, result);
		}
		++ cur;
	}

	// process the remaining vertices of the graph
	for (int_t vertex = 0; vertex < nVert; ++ vertex)
	{
		// debug
		if (newNum [vertex] > cur)
			throw "DEBUG: Wrong order 2.";

		// if the vertex has already been processed, skip it
		if (newNum [vertex] != cur)
			continue;

		// add the appropriate edges to the result graph
		result. addVertex ();
		addRenumEdges2 (g, vertex, newNum, numSets, cur,
			srcVert, result);
		++ cur;
	}

	// free memory and exit
	delete [] srcVert;
	delete [] newNum;
	return 0;
} /* collapseVertices2 */

/// Computes the graph that represents connections between a number
/// of the first vertices in the given graph.
/// The vertices of the result graph are the first "nVert" vertices
/// from the source graph. An edge is added to the new graph
/// iff there exists a path from one vertex to another,
/// without going through any other vertex in that set.
/// Runs DFS starting from each of the first "nVert" vertices,
/// and thus may be a little inefficient in some cases.
template <class wType>
inline int_t connectionGraph (const diGraph<wType> &g, int_t nVert,
	diGraph<wType> &result)
{
	// remember the number of vertices in the input graph
	int_t nVertG = g. countVertices ();
	if (!nVertG)
		return 0;

	// prepare a bitfield in which visited vertices will be marked
	chomp::homology::BitField visited;
	visited. allocate (nVertG);
	visited. clearall (nVertG);

	// run DFS for each of the starting vertices
	for (int_t startVertex = 0; startVertex < nVert; ++ startVertex)
	{
		// add this vertex to the resulting graph
		result. addVertex ();

		// prepare a counter and a stack of visited vertices
		int_t nVisited = 0;
		std::stack<int_t> s_visited;

		// prepare stacks for the DFS algorithm
		std::stack<int_t> s_vertex;
		std::stack<int_t> s_edge;

		// mark the starting vertex as visited
		visited. set (startVertex);
		s_visited. push (startVertex);
		++ nVisited;

		// use DFS to visit vertices reachable from that vertex
		int_t vertex = startVertex;
		int_t edge = 0;
		while (1)
		{
			// go back with the recursion
			// if all the edges have been processed
			if (edge >= g. countEdges (vertex))
			{
				// if this was the top recursion level
				// then quit
				if (s_vertex. empty ())
					break;

				// return from the recursion
				vertex = s_vertex. top ();
				s_vertex. pop ();
				edge = s_edge. top ();
				s_edge. pop ();
				continue;
			}

			// take the next vertex
			int_t next = g. getEdge (vertex, edge ++);

			// go to the deeper recursion level if not visited
			if (!visited. test (next))
			{
				// add an edge to the result if necessary
				if (next < nVert)
					result. addEdge (next);

				// store the previous variables at the stacks
				s_vertex. push (vertex);
				s_edge. push (edge);

				// take the new vertex
				vertex = next;
				edge = 0;

				// mark the new vertex as visited
				visited. set (vertex);
				s_visited. push (vertex);
				++ nVisited;
			}
		}

		// if this was the last vertex then skip the rest
		if (startVertex == nVert - 1)
			break;

		// mark each vertex as non-visited
		if (nVisited > (nVertG >> 6))
		{
			visited. clearall (nVertG);
		}
		else
		{
			while (!s_visited. empty ())
			{
				int_t vertex = s_visited. top ();
				s_visited. pop ();
				visited. clear (vertex);
			}
		}
	}

	// free memory and exit
	visited. free ();
	return 0;
} /* connectionGraph */

/// Computes the period of a strongly connected graph.
/// The period of a graph is defined as the greatest common divisor
/// of the lengths of all the cycles in the graph.
/// Period 1 means that the graph is aperiodic.
/// The complexity of this operation is linear (one DFS).
template <class GraphType>
inline int_t computePeriod (const GraphType &g)
{
	// remember the number of vertices in the input graph
	int_t nVertG = g. countVertices ();
	if (!nVertG)
		return 0;

	// prepare an array to record the depth of each visited vertex
	std::vector<int_t> visited (nVertG, 0);

	// run DFS starting at the first vertex
	// prepare stacks for the DFS algorithm
	std::stack<int_t> s_vertex;
	std::stack<int_t> s_edge;

	// mark the starting vertex as visited
	visited [0] = 1;

	// use DFS to visit vertices reachable from that vertex
	int_t vertex = 0;
	int_t edge = 0;
	int_t level = 1;
	int_t period = 0;
	while (1)
	{
		// go back with the recursion
		// if all the edges have been processed
		if (edge >= g. countEdges (vertex))
		{
			// if this was the top recursion level then quit
			if (s_vertex. empty ())
				break;

			// return from the recursion
			vertex = s_vertex. top ();
			s_vertex. pop ();
			edge = s_edge. top ();
			s_edge. pop ();
			-- level;
			continue;
		}

		// take the next vertex
		int_t next = g. getEdge (vertex, edge ++);

		// update the GCD of the cycle lengths
		if (visited [next])
		{
			// determine the period provided by the cycle
			int_t newPeriod = visited [next] - level - 1;
			if (newPeriod < 0)
				newPeriod = -newPeriod;

			// compute the GCD of the old and new periods
			int_t a = newPeriod;
			int_t b = period;
			while (b)
			{
				int_t t = b;
				b = a % b;
				a = t;
			}
			period = a;

			// if the graph turns out to be aperiodic
			// then immediately return the result
			if (period == 1)
				return period;
		}

		// go to the deeper recursion level if not visited
		else
		{
			// store the previous variables at the stacks
			s_vertex. push (vertex);
			s_edge. push (edge);

			// take the new vertex
			vertex = next;
			edge = 0;
			++ level;

			// mark the new vertex as visited
			visited [vertex] = level;
		}
	}

	// return the computed peirod and exit
	return period;
} /* computePeriod */

/// Computes the graph that represents flow-induced relations on Morse sets.
/// The vertices of the result graph are the first "nVert" vertices
/// from the source graph. An edge is added to the new graph
/// iff there exists a path from one vertex to another.
/// Edges that come from the transitive closure are not added.
template <class wType>
inline int_t inclusionGraph (const diGraph<wType> &g, int_t nVert,
	diGraph<wType> &result)
{
	// remember the number of vertices in the input graph
	int_t nVertG = g. countVertices ();
	if (!nVertG)
		return 0;

	// mark each vertex as non-visited
	chomp::homology::BitField visited;
	visited. allocate (nVertG);
	visited. clearall (nVertG);

	// create the sets of vertices reachable from each vertex
	chomp::homology::BitSets lists (nVertG, nVert);

	// prepare stacks for the DFS algorithm
	std::stack<int_t> s_vertex;
	std::stack<int_t> s_edge;

	// use DFS to propagate the reachable sets
	int_t startVertex = 0;
	int_t vertex = 0;
	int_t edge = 0;
	visited. set (vertex);
	while (1)
	{
		// go back with the recursion
		// if all the edges have been processed
		if (edge >= g. countEdges (vertex))
		{
			// if this was the top recursion level,
			// then find another starting point or quit
			if (s_vertex. empty ())
			{
				while ((startVertex < nVertG) &&
					(visited. test (startVertex)))
					++ startVertex;
				if (startVertex >= nVertG)
					break;
				vertex = startVertex;
				edge = 0;
				visited. set (vertex);
				continue;
			}

			// determine the previous vertex (to trace back to)
			int_t prev = s_vertex. top ();

			// add the list of the current vertex
			// to the list of the previous vertex
			// unless that was one of the first 'nVert' vertices
			if (vertex >= nVert)
				lists. addset (prev, vertex);

			// return from the recursion
			vertex = prev;
			s_vertex. pop ();
			edge = s_edge. top ();
			s_edge. pop ();
			continue;
		}

		// take the next vertex
		int_t next = g. getEdge (vertex, edge ++);

		// add the number to the list if appropriate
		if (next < nVert)
			lists. add (vertex, next);

		// go to the deeper recursion level if vertex not visited
		if (!visited. test (next))
		{
			// store the previous variables at the stacks
			s_vertex. push (vertex);
			s_edge. push (edge);

			// take the new vertex and mark as visited
			vertex = next;
			edge = 0;
			visited. set (vertex);
		}

		// add the list of the next vertex to the current one
		// if the vertex has already been visited before
		// and the vertex is not one of the first 'nVert' ones
		else if (next >= nVert)
		{
			lists. addset (vertex, next);
		}
	}

	// create the result graph based on the lists of edges
	for (int_t vertex = 0; vertex < nVert; ++ vertex)
	{
		result. addVertex ();
		int_t edge = lists. get (vertex);
		while (edge >= 0)
		{
			result. addEdge (edge);
			edge = lists. get (vertex, edge + 1);
		}
	}

	// free memory and exit
	visited. free ();
	return 0;
} /* inclusionGraph */

// --------------------------------------------------

/// A more complicated version of the 'inclusionGraph' function.
/// Computes the graph that represents flow-induced relations on Morse sets.
/// The vertices of the result graph are the first "nVert" vertices
/// from the source graph. An edge is added to the new graph
/// iff there exists a path from one vertex to another.
/// Edges that come from the transitive closure are not added.
/// Records vertices along connecting orbits using operator () with the
/// following arguments: source, target, vertex.
template<class wType, class TConn>
inline int_t inclusionGraph (const diGraph<wType> &g, int_t nVert,
	diGraph<wType> &result, TConn &connections)
{
	// remember the number of vertices in the input graph
	int_t nVertG = g. countVertices ();
	if (!nVertG)
		return 0;

	// mark each vertex as non-visited
	chomp::homology::BitField visited;
	visited. allocate (nVertG);
	visited. clearall (nVertG);

	// create the sets of vertices reachable from each vertex
	chomp::homology::BitSets forwardlists (nVertG, nVert);

	// prepare stacks for the DFS algorithm
	std::stack<int_t> s_vertex;
	std::stack<int_t> s_edge;

	// use DFS to propagate the reachable sets
	int_t startVertex = 0;
	int_t vertex = 0;
	int_t edge = 0;
	visited. set (vertex);
	while (1)
	{
		// go back with the recursion
		// if all the edges have been processed
		if (edge >= g. countEdges (vertex))
		{
			// if this was the top recursion level,
			// then find another starting point or quit
			if (s_vertex. empty ())
			{
				while ((startVertex < nVertG) &&
					(visited. test (startVertex)))
					++ startVertex;
				if (startVertex >= nVertG)
					break;
				vertex = startVertex;
				edge = 0;
				visited. set (vertex);
				continue;
			}

			// determine the previous vertex (to trace back to)
			int_t prev = s_vertex. top ();

			// add the list of the current vertex
			// to the list of the previous vertex
			// unless that was one of the first 'nVert' vertices
			if (vertex >= nVert)
				forwardlists. addset (prev, vertex);

			// return from the recursion
			vertex = prev;
			s_vertex. pop ();
			edge = s_edge. top ();
			s_edge. pop ();
			continue;
		}

		// take the next vertex
		int_t next = g. getEdge (vertex, edge ++);

		// add the number to the list if appropriate
		if (next < nVert)
			forwardlists. add (vertex, next);

		// go to the deeper recursion level if vertex not visited
		if (!visited. test (next))
		{
			// store the previous variables at the stacks
			s_vertex. push (vertex);
			s_edge. push (edge);

			// take the new vertex and mark as visited
			vertex = next;
			edge = 0;
			visited. set (vertex);
		}

		// add the list of the next vertex to the current one
		// if the vertex has already been visited before
		// and the vertex is not one of the first 'nVert' ones
		else if (next >= nVert)
		{
			forwardlists. addset (vertex, next);
		}
	}

	// ------------------------------------------

	// create the sets of vertices that can reach each vertex
	chomp::homology::BitSets backlists (nVertG, nVert);

	diGraph<wType> gT;
	g. transpose (gT);

	// do a simple test for debugging purposes
	if (!s_vertex. empty () || !s_edge. empty ())
		throw "DEBUG: Nonempty stacks in 'inclusionGraph'.";

	// mark all the vertices as unvisited for the backward run
	visited. clearall (nVertG);

	// use DFS to propagate the reachable sets
	startVertex = 0;
	vertex = 0;
	edge = 0;
	visited. set (vertex);
	while (1)
	{
		// go back with the recursion
		// if all the edges have been processed
		if (edge >= gT. countEdges (vertex))
		{
			// if this was the top recursion level,
			// then find another starting point or quit
			if (s_vertex. empty ())
			{
				while ((startVertex < nVertG) &&
					(visited. test (startVertex)))
					++ startVertex;
				if (startVertex >= nVertG)
					break;
				vertex = startVertex;
				edge = 0;
				visited. set (vertex);
				continue;
			}

			// determine the previous vertex (to trace back to)
			int_t prev = s_vertex. top ();

			// add the list of the current vertex
			// to the list of the previous vertex
			// unless that was one of the first 'nVert' vertices
			if (vertex >= nVert)
				backlists. addset (prev, vertex);

			// return from the recursion
			vertex = prev;
			s_vertex. pop ();
			edge = s_edge. top ();
			s_edge. pop ();
			continue;
		}

		// take the next vertex
		int_t next = gT. getEdge (vertex, edge ++);

		// add the number to the list if appropriate
		if (next < nVert)
			backlists. add (vertex, next);

		// go to the deeper recursion level if vertex not visited
		if (!visited. test (next))
		{
			// store the previous variables at the stacks
			s_vertex. push (vertex);
			s_edge. push (edge);

			// take the new vertex and mark as visited
			vertex = next;
			edge = 0;
			visited. set (vertex);
		}

		// add the list of the next vertex to the current one
		// if the vertex has already been visited before
		// and the vertex is not one of the first 'nVert' ones
		else if (next >= nVert)
		{
			backlists. addset (vertex, next);
		}
	}

	// ------------------------------------------

	// record the connections to the obtained data structure
	for (int_t v = nVert; v < nVertG; ++ v)
	{
		int_t bvertex = backlists. get (v);
		while (bvertex >= 0)
		{
			int_t fvertex = forwardlists. get (v);
			while (fvertex >= 0)
			{
				connections (bvertex, fvertex, v);
				fvertex = forwardlists. get (v, fvertex + 1);
			}
			bvertex = backlists. get (v, bvertex + 1);
		}
	}

	// ------------------------------------------
	
	// create the result graph based on the lists of edges
	for (int_t vertex = 0; vertex < nVert; ++ vertex)
	{
		result. addVertex ();
		int_t edge = forwardlists. get (vertex);
		while (edge >= 0)
		{
			result. addEdge (edge);
			edge = forwardlists. get (vertex, edge + 1);
		}
	}

	// free memory and exit
	visited. free ();
	return 0;
} /* inclusionGraph */

// --------------------------------------------------

/// Creates the adjacency matrix of the given graph.
/// m [i] [j] is set to 1 if the graph g contains the edge i -> j,
/// using the assignment operator.
template <class wType, class matrix>
inline void graph2matrix (const diGraph<wType> &g, matrix &m)
{
	int_t nVert = g. countVertices ();
	for (int_t v = 0; v < nVert; ++ v)
	{
		int_t nEdges = g. countEdges (v);
		for (int_t e = 0; e < nEdges; ++ e)
		{
			int_t w = g. getEdge (v, e);
			m [v] [w] = 1;
		}
	}
	return;
} /* graph2matrix */

/// Creates a graph based on the given adjacency matrix.
/// If m [i] [j] is nonzero, then the edge i -> j is added to the graph.
/// It is assumed that the graph g is initially empty.
/// The size of the matrix (the number of vertices), n, must be given.
template <class wType, class matrix>
inline void matrix2graph (const matrix &m, int_t n, diGraph<wType> &g)
{
	for (int_t v = 0; v < n; ++ v)
	{
		g. addVertex ();
		for (int_t w = 0; w < n; ++ w)
			if (m [v] [w])
				g. addEdge (w);
	}
	return;
} /* matrix2graph */

// --------------------------------------------------

/// Computes the transitive closure of an acyclic graph
/// defined by its adjacency matrix, using the Warshall's algorithm:
/// S. Warshall, A theorem on Boolean matrices, J. ACM 9 (1962) 11-12.
template <class matrix>
inline void transitiveClosure (matrix &m, int_t n)
{
	for (int_t k = 0; k < n; ++ k)
	{
		for (int_t i = 0; i < n; ++ i)
		{
			for (int_t j = 0; j < n; ++ j)
			{
				if (m [i] [k] && m [k] [j])
					m [i] [j] = 1;
			}
		}
	}
	return;
} /* transitiveClosure */

/// Computes the transitive reduction of a CLOSED acyclic graph
/// defined by its adjacency matrix, using the algorithm by D. Gries,
/// A.J. Martin, J.L.A. van de Snepscheut and J.T. Udding, An algorithm
/// for transitive reduction of an acyclic graph, Science of Computer
/// Programming 12 (1989), 151-155.
/// WARNING: The input graph MUST BE CLOSED, use the "transitiveClosure"
/// algorithm first if this is not the case.
template <class matrix>
inline void transitiveReduction (matrix &m, int_t n)
{
	for (int_t k = n - 1; k >= 0; -- k)
	{
		for (int_t i = 0; i < n; ++ i)
		{
			for (int_t j = 0; j < n; ++ j)
			{
				if (m [i] [k] && m [k] [j])
					m [i] [j] = 0;
			}
		}
	}
	return;
} /* transitiveReduction */

/// Computes the transitive reduction of an arbitrary acyclic graph.
/// The output graph must be initially empty.
template <class wType>
inline void transitiveReduction (const diGraph<wType> &g,
	diGraph<wType> &gRed)
{
	int_t nVert = g. countVertices ();
	if (nVert <= 0)
		return;
	chomp::homology::flatMatrix<char> m (nVert);
	m. clear (0);
	graph2matrix (g, m);
	cymealg::transitiveClosure (m, nVert);
	cymealg::transitiveReduction (m, nVert);
	matrix2graph (m, nVert, gRed);
	return;
} /* transitiveReduction */


} // namespace cymealg

#endif // _CYMEALG_VARIOUS_H_

/// @}

