/// @addtogroup test
/// @{

/////////////////////////////////////////////////////////////////////////////
///
/// \file
///
/// This is a simple program for testing selected algorithms
/// for the computation of the minimum cycle mean of a directed graph.
/// It reads the definition of a weighted graph from a text input file
/// and measures the time of the computation of the minimum cycle mean
/// using different algorithms.
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2020 by Pawel Pilarczyk.
//
// This file is part of my research software. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <http://www.gnu.org/licenses/>.

// Started on November 24, 2014. Last revision: August 23, 2019.


// include some standard C++ header files
#include <iostream>
#include <fstream>

// include selected header files from the CHomP library
#include "chomp/system/config.h"
#include "chomp/system/textfile.h"
#include "chomp/system/timeused.h"
#include "chomp/system/arg.h"
#include "chomp/struct/digraph.h"

// include relevant local header files
#include "cymealg.h"


// --------------------------------------------------
// -------------------- OVERTURE --------------------
// --------------------------------------------------

/// The title of the program and licensing information.
const char *title = "\
CyMeAlg Test 1.0. January 26, 2020. (C) 1997-2020 by Pawel Pilarczyk.\n\
This is free software. No warranty. Consult 'license.txt' for details.";

/// Brief help information on the program's usage.
const char *helpinfo = "\
This is a simple program for testing selected algorithms\n\
for the computation of the minimum cycle mean of a directed graph.\n\
It reads the definition of a weighted graph from a text input file\n\
and measures the time of the computation of the minimum cycle mean\n\
using different algorithms. The algorithms must be selected with the\n\
additional switches (see below); assumes '-m' if none provided.\n\
Call with:\n\
filename.txt - the name of a file that contains the definition of the graph.\n\
Switches and additional arguments:\n\
-k - run the classical Karp's algorithm,\n\
-kr - run the Karp's algorithm with correct rounding,\n\
-m - run the memory efficient algorithm,\n\
-mr - run the memory efficient algorithm with correct rounding,\n\
--log filename - save the output to a file (without progress indicators),\n\
--quiet - suppress data output to the screen (whcih can be still logged),\n\
--help - display this brief help information only and exit.\n\
For more information please consult the accompanying documentation\n\
or ask the program's author at http://www.PawelPilarczyk.com/.";

//-o - run an older version of the memory efficient algorithm,
//-or - run the older version with correct rounding,


// --------------------------------------------------
// ---------------------- test ----------------------
// --------------------------------------------------

void cymealgTest (const char *filename, bool runKarp, bool runKarpRnd,
	bool runMem, bool runMemRnd, bool runOld, bool runOldRnd)
{
	using chomp::homology::timeused;
	using chomp::homology::sout;

	cymealg::diGraph<double> g;
	chomp::homology::diGraph<double> gOld;
	if (runKarp || runKarpRnd || runMem || runMemRnd)
	{
		sout << "Reading a graph from '" << filename << "'... ";
		timeused readingTime;
		std::ifstream in (filename);
		if (!in)
			chomp::homology::fileerror (filename, "open");
		in >> g;
		sout << readingTime << ".\n";
		sout << "The graph has " << g. countVertices () <<
			" vertices and " << g. countEdges () << " edges.\n";
	}
	if (runOld || runOldRnd)
	{
		sout << "Reading a graph from '" << filename <<
			"' to the old data structure... ";
		timeused readingTime;
		std::ifstream in (filename);
		if (!in)
			chomp::homology::fileerror (filename, "open");
		cymealg::read (in, gOld);
		sout << readingTime << ".\n";
		sout << "The graph has " << gOld. countVertices () <<
			" vertices and " << gOld. countEdges () <<
			" edges.\n";
	}

	const cymealg::tBoostRounding<double> rigorousRounding;
	const cymealg::dummyRounding<double> approximateRounding;

	sout << "Computing the minimum mean cycle weight:\n";
	sout << "========================================\n";

	double resultKarp (0);
	if (runKarp)
	{
		timeused timeRun;
		sout << "Running the 'classical' Karp's algorithm "
			"with default rounding...\n";
		cymealg::diGraph<double> *transposed = 0;
		resultKarp = minMeanCycleWeight (g, transposed);
		sout << "Result = " << resultKarp <<
			", computed in " << timeRun << ".\n";
	}

	double resultKarpRnd (0);
	if (runKarpRnd)
	{
		timeused timeRun;
		sout << "Running the 'classical' Karp's algorithm "
			"with rigorous rounding...\n";
		cymealg::diGraph<double> *transposed = 0;
		resultKarpRnd = minMeanCycleWeight
			(g, rigorousRounding, transposed);
		sout << "Result = " << resultKarpRnd <<
			", computed in " << timeRun << ".\n";
	}

	double resultMem (0);
	if (runMem)
	{
		timeused timeRun;
		sout << "Running the memory efficient algorithm "
			"with default rounding...\n";
		cymealg::diGraph<double> *transposed = 0;
		resultMem = minMeanCycleWeightMem
			(g, approximateRounding, transposed);
		sout << "Result = " << resultMem <<
			", computed in " << timeRun << ".\n";
	}

	double resultMemRnd (0);
	if (runMemRnd)
	{
		timeused timeRun;
		sout << "Running the memory efficient algorithm "
			"with rigorous rounding...\n";
		cymealg::diGraph<double> *transposed = 0;
		resultMemRnd = minMeanCycleWeightMem
			(g, rigorousRounding, transposed);
		sout << "Result = " << resultMemRnd <<
			", computed in " << timeRun << ".\n";
	}

	double resultOld (0);
	if (runOld)
	{
		timeused timeRun;
		sout << "Running an old memory efficient algorithm "
			"with approximate rounding...\n";
		chomp::homology::diGraph<double> *transposed = 0;
		resultOld = gOld. minMeanCycleWeightMem
			(approximateRounding, transposed);
		sout << "Result = " << resultOld <<
			", computed in " << timeRun << ".\n";
	}

	double resultOldRnd (0);
	if (runOldRnd)
	{
		timeused timeRun;
		sout << "Running an old memory efficient algorithm "
			"with rigorous rounding...\n";
		chomp::homology::diGraph<double> *transposed = 0;
		resultOldRnd = gOld. minMeanCycleWeightMem
			(rigorousRounding, transposed);
		sout << "Result = " << resultOldRnd <<
			", computed in " << timeRun << ".\n";
	}

//	sout << g;

	return;
} /* cymealgTest */

// --------------------------------------------------
// ---------------------- main ----------------------
// --------------------------------------------------

/// The main procedure of the program.
/// Returns: 0 = Ok, -1 = Error, 1 = Help displayed, 2 = Wrong arguments.
int main (int argc, char *argv [])
{
	using chomp::homology::sout;
	using chomp::homology::slog;
	using chomp::homology::setstreams;
	using chomp::homology::program_time;
	using chomp::homology::commandline;
	using chomp::homology::currenttime;

	// turn on a message that will appear if the program does not finish
	chomp::homology::program_time = "Aborted after";

	// prepare user-configurable data
	char *filename (0);
	bool runKarp (false);
	bool runKarpRnd (false);
	bool runMem (false);
	bool runMemRnd (false);
	bool runOld (false);
	bool runOldRnd (false);

	// analyze the command line
	chomp::homology::arguments a;
	arg (a, NULL, filename);
	argswitch (a, "kr", runKarpRnd, true);
	argswitch (a, "k", runKarp, true);
	argswitch (a, "mr", runMemRnd, true);
	argswitch (a, "m", runMem, true);
	argswitch (a, "or", runOldRnd, true);
	argswitch (a, "o", runOld, true);
	arghelp (a);

	argstreamprepare (a);
	int argresult = a. analyze (argc, argv);
	argstreamset ();

	// show the program's title
	if (argresult >= 0)
		sout << title << '\n';

	// if something was incorrect, show an additional message and exit
	if (argresult < 0)
	{
		sout << "Call with '--help' for help.\n";
		return 2;
	}

	// if help requested or no filename present, show help information
	if ((argresult > 0) || (!filename || !*filename))
	{
		sout << helpinfo << '\n';
		return 1;
	}

	// try running the main function and catch an error message if thrown
	try
	{
		// run the test with the given filename
		if (!runKarp && !runKarpRnd && !runMem && !runMemRnd &&
			!runOld && !runOldRnd)
		{
			runMem = true;
		}
		cymealgTest (filename, runKarp, runKarpRnd,
			runMem, runMemRnd, runOld, runOldRnd);

		program_time = "Total time used:";
		program_time = 1;
		return 0;
	}
	catch (const char *msg)
	{
		sout << "ERROR: " << msg << '\n';
		return -1;
	}
	catch (const std::exception &e)
	{
		sout << "ERROR: " << e. what () << '\n';
		return -1;
	}
	catch (...)
	{
		sout << "ABORT: An unknown error occurred.\n";
		return -1;
	}
} /* main */


/// @}

